/*
 *	head-cn50xx.S  -- In kernel Octeon CN50XX startup code
 */

	/*
	 * Enable 64bit mode and accesses.
	 */
	dli     v0, 0x500000e5          /* Enable COP's, 64bit, INTRS */
	mtc0    v0, $12, 0              /* Write COP0_STATUS_REG */
	nop

	/*
	 * Only core 0 runs the RAM hardware setup code. All other cores
	 * spin until told to start the kernel proper.
	 */
	mfc0	t0, $15, 1		/* Get EBASE register */
	andi	t0, 0xff		/* Just the num cores field */
	bnez	t0, coreXreset		/* If zero then boot core */

	/*
	 * DRAM initialization code.
	 */
	dli	t0, 0x8001180088000000	/* Base LMC memory regs address */
	dli	t2, 0x8001180080000000	/* Base of the L2C registers */

	/* DCLK Initialization sequence */
	dli	t1, 0x00000000617c0e58  /* Reset memory controller */
	sd	t1, 0x0010(t0)		/* Write LMC0_CTL reg */

	dli	t1, 0x0000000050c20401	/* QDLL_ENA turned off */
	sd	t1, 0x0018(t0)		/* Write LMC0_DDR2_CTL reg */

	dli	t1, 0x00000000080fc104	/* PLL controler, RESET_N=0 */
	sd	t1, 0x00a8(t0)		/* Write LMC0_PLL_CTL reg */

	ld	t3, 0x0780(t2)		/* Read LD2_BST0 reg */
	dli	t3, 0x800		/* Delay loop count 2048 */
1:	daddi	t3, t3, -1		/* Decrement loop counter */
	bnezl	t3, 1b

	dli	t1, 0x000000000c0fc104	/* PLL controler, set RESET_N=1 */
	sd	t1, 0x00a8(t0)		/* Write LMC0_PLL_CTL reg */

	ld	t3, 0x0780(t2)		/* Read LD2_BST0 reg */
	dli	t3, 0x800		/* Delay loop count 2048 */
1:	daddi	t3, t3, -1		/* Decrement loop counter */
	bnezl	t3, 1b

	dli	t1, 0x00000000040fc104	/* PLL controler, run */
	sd	t1, 0x00a8(t0)		/* Write LMC0_PLL_CTL reg */

	ld	t3, 0x0780(t2)		/* Read LD2_BST0 reg */

	/* DRESET Initialization Sequence */
	dli	t1, 0x00000000617c0e58  /* Reset memory controller */
	sd	t1, 0x0010(t0)		/* Write LMC0_CTL reg */

	dli	t1, 0x0000000050c20401	/* QDLL_ENA turned off */
	sd	t1, 0x0018(t0)		/* Write LMC0_DDR2_CTL reg */
	dli	t1, 0x0000000050c20501	/* QDLL_ENA enabled */
	sd	t1, 0x0018(t0)		/* Write LMC0_DDR2_CTL reg */

	ld	t3, 0x0780(t2)		/* Read LD2_BST0 reg */
	dli	t3, 0x800		/* Delay loop count 2048 */
1:	daddi	t3, t3, -1		/* Decrement loop counter */
	bnezl	t3, 1b

	dli	t1, 0x00000000617c0658  /* DRESET disabled */
	sd	t1, 0x0010(t0)		/* Write LMC0_CTL reg */

	ld	t3, 0x0780(t2)		/* Read LD2_BST0 reg */
	dli	t3, 0x100		/* Delay loop count */
1:	daddi	t3, t3, -1		/* Decrement loop counter */
	bnezl	t3, 1b

	/* LMC Initialization Sequence */
	dli	t1, 0x0000000000010001
	sd	t1, 0x0030(t0)		/* Write LMC0_WODT_CTL reg */
	dli	t1, 0x0000000000000000
	sd	t1, 0x0078(t0)		/* Write LMC0_RODT_CTL reg */
	dli	t1, 0x0000000000010207
	sd	t1, 0x00a0(t0)		/* Write LMC0_RODT_COMP_CTL reg */
	dli	t1, 0x0000000000030000
	sd	t1, 0x0098(t0)		/* Write LMC0_DUAL_MEMCFG reg */
	dli	t1, 0x00000000617c0658  /* Final controller setup */
	sd	t1, 0x0010(t0)		/* Write LMC0_CTL reg */
	dli	t1, 0x0000000000000200
	sd	t1, 0x0090(t0)		/* Write LMC0_CTL1 reg */
	dli	t1, 0x00000000388e848b
	sd	t1, 0x0008(t0)		/* Write LMC0_MEM_CFG1 reg */
	dli	t1, 0x0000000050c20501
	sd	t1, 0x0018(t0)		/* Write LMC0_DDR2_CTL reg */
	dli	t1, 0x0000000000000800
	sd	t1, 0x0000(t0)		/* Write LMC0_MEM_CFG0 reg */

	dli	t1, 0x0000000000000801	/* INIT_START set */
	sd	t1, 0x0000(t0)		/* Write LMC0_MEM_CFG0 reg */

	ld	t3, 0x0780(t2)		/* Read LD2_BST0 reg */
	dli	t3, 0x100		/* Delay loop count */
1:	daddi	t3, t3, -1		/* Decrement loop counter */
	bnezl	t3, 1b

	/*
	 * Copy the kernel binary from flash to RAM. We start by copying
	 * the base 4k of the kernel (which is this init code we are running
	 * right now), and then transfering execution to cached RAM.
	 * Completing the 3MB copy from cached RAM is _alot_ quicker.
	 */
	dli	t0, 0xffffffffbfc00000	/* Base of boot loader in flash */
	dli	t1, 0x1000		/* Copy just this base code first */
	dli	t2, 0xffffffff80100000	/* Copy boot loader to RAM address */
copybaseloop:
	ld	t3, 0(t0)		/* Load dword from flash */
	sd	t3, 0(t2)		/* Store it in RAM */
	dsubu	t1, t1, 8		/* Decrement counter */
	daddiu	t0, t0, 8		/* Increment flash pointer */
	daddiu	t2, t2, 8		/* Increment RAM pointer */
	bnez	t1, copybaseloop	/* Loop until all copied */

	/*
	 * Transfer execution to this code in cached RAM.
	 */
	dla	t1, copybasedone
	j	t1			/* Jump into RAM */

copybasedone:
	nop

	/*
	 * Now copy the rest of the kernel binary into RAM.
	 */
	dli	t1, (0x300000 - 0x1000)	/* Copy rest of kernel */
copyloop:
	ld	t3, 0(t0)		/* Load dword from flash */
	sd	t3, 0(t2)		/* Store it in RAM */
	dsubu	t1, t1, 8		/* Decrement counter */
	daddiu	t0, t0, 8		/* Increment flash pointer */
	daddiu	t2, t2, 8		/* Increment RAM pointer */
	bnez	t1, copyloop		/* Loop until all copied */

	dli	sp, (128 * 1024 - 8)	/* Set an initial stack */

	/*
	 * Just setup the kernel args now and we are all done.
	 */
	dli	a0, 0			/* argc = 0 */
	dli	a1, 0			/* argv = NULL */
	dli	a2, 1			/* Init core */
	dli	a3, 0			/* Boot descriptors */
	dla	t0, kernel_entry	/* Start address of kernel image */
	sd	t0, 0(zero)		/* Marker for other cores */

	dli	t0, 0x8001070000000700	/* Take other CPU cores out of reset */
	sd	zero, 0(t0)

	j	kernel_entry		/* Jump into kernel proper */

/****************************************************************************/

/*
 * Non-boot cores enter here (so all cores other than CPU 0). They spin
 * here waiting to be told that they can jump into the kenel. RAM will
 * be setup and working by the time we get here.
 */
coreXreset:
	dla	t0, coreXwait		/* Load real RAM address */
	j	t0			/* Jump into RAM */

coreXwait:
	ld	t0, 0(zero)		/* Check for start vector */
	beqz	t0, coreXwait		/* If no vector keep spinning */

coreXboot:
	dli	a0, 0			/* argc = 0 */
	dli	a1, 0			/* argv = NULL */
	dli	a2, 0			/* Secondary core */
	dli	a3, 0			/* Boot descriptors */

	j	kernel_entry		/* Jump into kernel proper */

/****************************************************************************/
