/****************************************************************************/

/*
 *	m5272sim.h -- ColdFire 5272 System Integration Module support.
 *
 *	(C) Copyright 1999, Greg Ungerer (gerg@snapgear.com)
 * 	(C) Copyright 2000, Lineo Inc. (www.lineo.com) 
 */

/****************************************************************************/
#ifndef	m5272sim_h
#define	m5272sim_h
/****************************************************************************/

/*
 *	Define the 5272 SIM register set addresses.
 */
#define	MCFSIM_SCR		0x04		/* SIM Config reg (r/w) */
#define	MCFSIM_SPR		0x06		/* System Protection reg (r/w)*/
#define	MCFSIM_PMR		0x08		/* Power Management reg (r/w) */
#define	MCFSIM_APMR		0x0e		/* Active Low Power reg (r/w) */
#define	MCFSIM_DIR		0x10		/* Device Identity reg (r/w) */

#define	MCFSIM_ICR1		0x20		/* Intr Ctrl reg 1 (r/w) */
#define	MCFSIM_ICR2		0x24		/* Intr Ctrl reg 2 (r/w) */
#define	MCFSIM_ICR3		0x28		/* Intr Ctrl reg 3 (r/w) */
#define	MCFSIM_ICR4		0x2c		/* Intr Ctrl reg 4 (r/w) */

#define MCFSIM_ISR		0x30		/* Interrupt Source reg (r/w) */
#define MCFSIM_PITR		0x34		/* Interrupt Transition (r/w) */
#define	MCFSIM_PIWR		0x38		/* Interrupt Wakeup reg (r/w) */
#define	MCFSIM_PIVR		0x3f		/* Interrupt Vector reg (r/w( */

#define	MCFSIM_WRRR		0x280		/* Watchdog reference (r/w) */
#define	MCFSIM_WIRR		0x284		/* Watchdog interrupt (r/w) */
#define	MCFSIM_WCR		0x288		/* Watchdog counter (r/w) */
#define	MCFSIM_WER		0x28c		/* Watchdog event (r/w) */

#define	MCFSIM_CSBR0		0x40		/* CS0 Base Address (r/w) */
#define	MCFSIM_CSOR0		0x44		/* CS0 Option (r/w) */
#define	MCFSIM_CSBR1		0x48		/* CS1 Base Address (r/w) */
#define	MCFSIM_CSOR1		0x4c		/* CS1 Option (r/w) */
#define	MCFSIM_CSBR2		0x50		/* CS2 Base Address (r/w) */
#define	MCFSIM_CSOR2		0x54		/* CS2 Option (r/w) */
#define	MCFSIM_CSBR3		0x58		/* CS3 Base Address (r/w) */
#define	MCFSIM_CSOR3		0x5c		/* CS3 Option (r/w) */
#define	MCFSIM_CSBR4		0x60		/* CS4 Base Address (r/w) */
#define	MCFSIM_CSOR4		0x64		/* CS4 Option (r/w) */
#define	MCFSIM_CSBR5		0x68		/* CS5 Base Address (r/w) */
#define	MCFSIM_CSOR5		0x6c		/* CS5 Option (r/w) */
#define	MCFSIM_CSBR6		0x70		/* CS6 Base Address (r/w) */
#define	MCFSIM_CSOR6		0x74		/* CS6 Option (r/w) */
#define	MCFSIM_CSBR7		0x78		/* CS7 Base Address (r/w) */
#define	MCFSIM_CSOR7		0x7c		/* CS7 Option (r/w) */

#define	MCFSIM_SDCR		0x180		/* SDRAM Configuration (r/w) */
#define	MCFSIM_SDTR		0x184		/* SDRAM Timing (r/w) */
#define	MCFSIM_DCAR0		0x4c		/* DRAM 0 Address reg(r/w) */
#define	MCFSIM_DCMR0		0x50		/* DRAM 0 Mask reg (r/w) */
#define	MCFSIM_DCCR0		0x57		/* DRAM 0 Control reg (r/w) */
#define	MCFSIM_DCAR1		0x58		/* DRAM 1 Address reg (r/w) */
#define	MCFSIM_DCMR1		0x5c		/* DRAM 1 Mask reg (r/w) */
#define	MCFSIM_DCCR1		0x63		/* DRAM 1 Control reg (r/w) */

#define	MCFSIM_PACNT_OFF	0x80		/* Port A Control (r/w) */
#define	MCFSIM_PADDR_OFF	0x84		 /* Port A Direction (r/w) */
#define	MCFSIM_PADAT_OFF	0x86		 /* Port A Data (r/w) */
#define	MCFSIM_PBCNT_OFF	0x88		 /* Port B Control (r/w) */
#define	MCFSIM_PBDDR_OFF	0x8c		 /* Port B Direction (r/w) */
#define	MCFSIM_PBDAT_OFF	0x8e		 /* Port B Data (r/w) */
#define	MCFSIM_PCDDR_OFF	0x94		 /* Port C Direction (r/w) */
#define	MCFSIM_PCDAT_OFF	0x96		 /* Port C Data (r/w) */
#define	MCFSIM_PDCNT_OFF	0x98		 /* Port D Control (r/w) */

#define	MCFSIM_PACNT		(MCF_MBAR + MCFSIM_PACNT_OFF)
#define	MCFSIM_PADDR		(MCF_MBAR + MCFSIM_PADDR_OFF)
#define	MCFSIM_PADAT		(MCF_MBAR + MCFSIM_PADAT_OFF)
#define	MCFSIM_PBCNT		(MCF_MBAR + MCFSIM_PBCNT_OFF)
#define	MCFSIM_PBDDR		(MCF_MBAR + MCFSIM_PBDDR_OFF)
#define	MCFSIM_PBDAT		(MCF_MBAR + MCFSIM_PBDAT_OFF)
#define	MCFSIM_PCDDR		(MCF_MBAR + MCFSIM_PCDDR_OFF)
#define	MCFSIM_PCDAT		(MCF_MBAR + MCFSIM_PCDAT_OFF)
#define	MCFSIM_PDCNT		(MCF_MBAR + MCFSIM_PDCNT_OFF)

/*
 *	Define system peripheral IRQ usage.
 */
#define	MCFINT_VECBASE		64		/* Base of interrupts */
#define	MCF_IRQ_SPURIOUS	64		/* User Spurious */
#define	MCF_IRQ_EINT1		65		/* External Interrupt 1 */
#define	MCF_IRQ_EINT2		66		/* External Interrupt 2 */
#define	MCF_IRQ_EINT3		67		/* External Interrupt 3 */
#define	MCF_IRQ_EINT4		68		/* External Interrupt 4 */
#define	MCF_IRQ_TIMER1		69		/* Timer 1 */
#define	MCF_IRQ_TIMER2		70		/* Timer 2 */
#define	MCF_IRQ_TIMER3		71		/* Timer 3 */
#define	MCF_IRQ_TIMER4		72		/* Timer 4 */
#define	MCF_IRQ_UART1		73		/* UART 1 */
#define	MCF_IRQ_UART2		74		/* UART 2 */
#define	MCF_IRQ_PLIP		75		/* PLIC 2Khz Periodic */
#define	MCF_IRQ_PLIA		76		/* PLIC Asynchronous */
#define	MCF_IRQ_USB0		77		/* USB Endpoint 0 */
#define	MCF_IRQ_USB1		78		/* USB Endpoint 1 */
#define	MCF_IRQ_USB2		79		/* USB Endpoint 2 */
#define	MCF_IRQ_USB3		80		/* USB Endpoint 3 */
#define	MCF_IRQ_USB4		81		/* USB Endpoint 4 */
#define	MCF_IRQ_USB5		82		/* USB Endpoint 5 */
#define	MCF_IRQ_USB6		83		/* USB Endpoint 6 */
#define	MCF_IRQ_USB7		84		/* USB Endpoint 7 */
#define	MCF_IRQ_DMA		85		/* DMA Controller */
#define	MCF_IRQ_ERX		86		/* Ethernet Receiver */
#define	MCF_IRQ_ETX		87		/* Ethernet Transmitter */
#define	MCF_IRQ_ENTC		88		/* Ethernet Non-Time Critical */
#define	MCF_IRQ_QSPI		89		/* Queued Serial Interface */
#define	MCF_IRQ_EINT5		90		/* External Interrupt 5 */
#define	MCF_IRQ_EINT6		91		/* External Interrupt 6 */
#define	MCF_IRQ_SWTO		92		/* Software Watchdog */
#define	MCFINT_VECMAX		95		/* Maxmum interrupt */

#define	MCF_IRQ_TIMER		MCF_IRQ_TIMER1
#define	MCF_IRQ_PROFILER	MCF_IRQ_TIMER2

/*
 * Generic GPIO support
 */
#define MCFGPIO_PIN_MAX			48
#define MCFGPIO_IRQ_MAX			-1
#define MCFGPIO_IRQ_VECBASE		-1

/****************************************************************************/

#if defined(__ASSEMBLER__) && defined(CONFIG_ROM)

.macro m5272_setup
	movel	#(MCF_MBAR + 1), %a0		/* Set MBAR address */
	movec	%a0, %MBAR
	subql	#1, %a0				/* Back to real MBAR addr */

	moveb	#0x4f, %d0			/* Set vector base to 64 */
	moveb	%d0, MCFSIM_PIVR(%a0)

	movel	#0x20000001, %d0		/* Set internal RAM address */
	movec	%d0, %RAMBAR0

	CACHE_ENABLE

	/*
	 *	Set up the device chip selects and SDRAM controller.
	 */
	movel	#0xf0000201, %d0		/* Map flash to 0xf0000000 */
	movel	%d0, MCFSIM_CSBR0(%a0)		/* Set address and width */
	movel	#0xf0000014, %d0		/* Set address mask */
	movel	%d0, MCFSIM_CSOR0(%a0)

	movel	#0x30600201, %d0		/* Map Davicom to 0x30600000 */
	movel	%d0, MCFSIM_CSBR3(%a0)		/* Set address and width */
	movel	#0xfff0007c, %d0		/* Set address mask */
	movel	%d0, MCFSIM_CSOR3(%a0)

	movel	#0x00000601, %d0		/* Map RAM to 0x00000000 */
	movel	%d0, MCFSIM_CSBR7(%a0)		/* Set address and width */
	movel	#0xff80017c, %d0		/* Set address mask */
	movel	%d0, MCFSIM_CSOR7(%a0)

	movel	#0x0000f539, %d0		/* Set SDRAM mode bits */
	movel	%d0, MCFSIM_SDTR(%a0)
	movel	#0x00002211, %d0
	movel	%d0, MCFSIM_SDCR(%a0)

	/*
	 *	Setup the devices multi-function pins to match the board.
	 *	Initialize the GPIO pins as needed too.
	 */
	movel	#0x00000000, %d0		/* Set multifunction pins */
	movel	%d0, MCFSIM_PACNT_OFF(%a0)	/* Set port A bits first */
	movew	#0x001f, %d0
	movew	%d0, MCFSIM_PADDR_OFF(%a0)
	movew	#0xffff, %d0
	movew	%d0, MCFSIM_PADAT_OFF(%a0)

	movel	#0x55550555, %d0		/* Now set port B bits */
	movel	%d0, MCFSIM_PBCNT_OFF(%a0)
	movew	#0x0040, %d0
	movew	%d0, MCFSIM_PBDDR_OFF(%a0)
	movew	#0xffff, %d0
	movew	%d0, MCFSIM_PBDAT_OFF(%a0)

	movel	#0x00000000, %d0		/* Now set port D bits */
	movel	%d0, MCFSIM_PDCNT_OFF(%a0)

	/*
	 *	Move .data and .init regions from flash to RAM.
	 */
	lea	_etext, %a0			/* Start of data in flash */
	lea	_sdata, %a1			/* Address of data region */
	lea	_edata, %a2			/* End of data region */
_copy_data:
	movel	(%a0)+, %d0			/* Next word from flash */
	movel	%d0, (%a1)+			/* Copy it to RAM */
	cmpl	%a1, %a2			/* Check if all copied */
	bne	_copy_data

	lea	__init_begin, %a1		/* Address of init region */
	lea	__init_end, %a2			/* End of init region */
_copy_init:
	movel	(%a0)+, %d0			/* Next word from flash */
	movel	%d0, (%a1)+			/* Copy it to RAM */
	cmpl	%a1, %a2			/* Check if all copied */
	bne	_copy_init

.endm

#define	PLATFORM_SETUP	m5272_setup

#endif /* __ASSEMBLER__ && CONFIG_ROM */

/****************************************************************************/
#endif	/* m5272sim_h */
